/*  
                                 FIRESTEM II

Adapted from the 1-D Firestem model by Joshua Jones.
Original 2-D cylindrical implementation by Ravishankar Subramaniyan
Modifications and additions by Anthony Bova are indicated by 'asb'  in the comments.
Modifications and additions by Efthalia Chatziefstratiou are indicated by 'ec'  in the comments.

To Compile the code:
   g++ -O3 -fexceptions -LBLAS -I../Include -I../../AMD/Include -I../../UFconfig -o test stem_2d.cpp ../Lib/libumfpack.a ../../AMD/Lib/libamd.a -lm 
   On asb's laptop: g++ -O3 -lblas -I/usr/lib -I/usr/include -I/usr/include/suitesparse -o Executable_Name FireStemCodeFile.cpp /usr/lib/libumfpack.a /usr/lib/libamd.a -lm
   -- call from inside 2Dheat folder in UMFPACK
   
   REFERENCES:Efthalia Chatziefstratiou, Gil Bohrer, Anthony Bova, Ravishankar Subramanian, Renato Frasson, Amy Scherzer, Bret Butler, Matthew Dickinson (2013) FireStme2D - atwo-dimensional heat transfer model for simulating tre stem injury in fires. PLOS ONE.
   
*/

#include <iostream>
#include <fstream>
#include "math.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "umfpack.h" // header file for sparse matrix
//#define Dr 0.001 // discrete radial units (m) -- asb 24 Oct10
#define ROW_ELEMENTS 5 // max of elements in a row in the sparse matrix
#define MOVIE_FRAMES 300 // # of movie frames for T_movie output file - asb

// Constants added by asb
#define k_water 6.05E+5 //constant for moisture loss rate function [K^.5 / s]
#define E_wateroverR 5956 //Activation energy over R for moisture loss function [K]
#define E_pyroverR 15610 //Ragland & Aerts 1991 BioRes. Tech. 37:161-168.  E/R=3.1e4 cal/mol / 1.9859 cal/mol K
#define A_pyr  7e7 //ibid. Apyr= 2.52e11 hr-1  * 1 hr/3600 s
#define T_crit 341// 273 K + 61 C, from Dickinson and Johnson 2004
#define DeltaH 76035 // Dickinson and Johnson 2004
#define b_comp 61.62// Dickinson and Johnson 2004 (per compemsation law in Rosenberg et al. Nature 471-73, 1971)
#define R_mol 1.987//
#define kB_over_h 2.08e10 //

using namespace std;

int main(int argc,char *argv[]) 
{

  if(argc<4)
    {
      printf("Usage: <program name> <boundary Condition name> <tree species number>\n");
      printf("0-Chestnut Oak\n1-Douglas-fir\n2-Juvenile Red Maple\n3-Juvenile Chestnut Oak\n4-Ponderosa Pine\n5-Red Maple\n");
      exit(0);
    }

  char m_FileName[35]=" " ;
  int m_speciesname=0,charcount=0;

  strcpy(m_FileName,argv[1]); // Takes the boundary condition file
  charcount=0;

  m_speciesname=atoi(argv[2]); // Species name as input
  switch(m_speciesname)
    {
    case 0:
      printf("The Species selected is Chestnut Oak\n");
      break;
    case 1:
      printf("The Species selected is Douglas-Fir\n");
      break;
    case 2:
      printf("The Species selected is Juvenile Red Maple\n");
      break;
    case 3:
      printf("The Species selected is Juvenile Chestnut Oak\n");
      break;
    case 4:
      printf("The Species selected is Ponderosa Pine\n");
      break;
    case 5:
      printf("The Species selected is Red Maple\n");
      break;
    case 6:
      printf("The Species selected is Hickory\n");
      break;
    default:
      printf("The species entered is not valid\n");
      exit(1);
    }

  double m_radius = 0.14605,
    m_thickness = 0.0128,
    m_moisture = 0.84,
    Dr=0.001, // -ec
    m_flux = 1.0, 
    m_inittemp = 17,
    //barkexp=0,
    expfactor=0,
    tcdepth=0,
    Tsat=0,
    //elevation=10000,
    //pressure=0,
    //temp_dummy=273.0,
    //water_dummy=0,
    //vap_enthalpy=2265000,
    inoverout=0,// asb
    I_moisture = 0., // Initial Inner bark moisture 
    O_moisture=0., // Initial outer bark moisture
    W_moisture=0., // Initial wood moisture
    W_mult=0.00, //Multiplier for rate of water loss
    P_mult=0., //Multiplier for rate of pyrolysis
    //double DeltaH=0.,// Dickinson and Johnson 2004
    rhobark=0., 
    rhowood=0.,
    innerbark=0., // Inner bark thickness
    t_step=.1; //  added 02Oct10 asb for adjustable time step routine
    
	
  //double flux_input=0;
  int m_numnode = 0,//NUM_NODES, // Defines the # of nodes along the radial line
    //m_hour = 18,	
    m_Radio = 1,
     m_numtheta=0; //Defines number of 'wedges' in polar coordinate domain

  int tcloc=0; //Location (radial depth) of thermocouple in experiments 
  //int radflag=0;
  int movie_index=0;
  int iterations=0;
  int seconds=0;
  //int moisture_flag=0;
  //int moisture_calc_flag=0;
  ifstream bndycond(m_FileName);

  /*
    Reads the radius of the stem, thickness of the bark, moisture content(in percentage) and other
    details from the boundary conditions file
  */

 //Read initial variable values from input file
  bndycond>>m_radius>>m_thickness>>innerbark>>Dr>>m_inittemp>>rhobark>>rhowood>>O_moisture>>I_moisture>>W_moisture>>m_numtheta>>tcdepth>>seconds>>t_step>>W_mult>>P_mult;//-ec
 
 cout<<"inner:"<<innerbark<<endl;
 

 // ===========

 W_mult  = atoi(argv[3]);  // atoi_ convert string to integer - ec

 // ===========


m_moisture = I_moisture; //Inner moisture is default moisture used in moisture profile calculations -asb
m_numnode = int(m_radius/Dr);//calculate number of nodes needed -asb 


if(innerbark>0.) inoverout=(innerbark / (m_thickness-innerbark)); //Calculate ratio of inner to outer bark
 
  
  /*
  //Uncomment this section if printout is needed

  cout << "mradius " << m_radius <<endl;
  cout << "mthickness " << m_thickness <<endl;
  cout << "minittemp " << m_inittemp <<endl;
  cout << "Omoisture " << O_moisture <<endl;
  cout << "Imoisture " << I_moisture <<endl;
  cout << "Wmoisture " << W_moisture <<endl;
  cout << "tcloc " << tcloc <<endl;
  cout << "tcdepth " << tcdepth <<endl;
  cout << "time step " << t_step <<endl;
  cout << "seconds " << seconds <<endl;
  cout << "num_nodes " << m_numnode -1  <<endl;
  cout << "num_theta " << m_numtheta <<endl;
*/

  double **nodes=new double *[m_numnode+2];
  double **temps=new double *[m_numnode+2];

  movie_index=iterations/MOVIE_FRAMES;// FOR TEMPERATURE MOVIE OUTPUT FILE

  //Distance between 2 control volumes
  double **Dx=new double *[m_numnode+2];

  //Distance between 2 nodes
  double **dx=new double *[m_numnode+2];

  double **cntrlsrfcs=new double *[m_numnode+2];

  //Thermal conditivity at the control surfaces
  double **kcs=new double *[m_numnode+2];
  //density
  double **rho=new double *[m_numnode+2];
  //specific heat
  double **Cp=new double *[m_numnode+2];
  double **k=new double *[m_numnode+2];

  double **mass=new double *[m_numnode+2];
  double **volatilemass=new double *[m_numnode+2];
  double **hightemps=new double *[m_numnode+2];

  double **oldV=new double *[m_numnode+2];
  //	double **energyconservation=new double *[m_numnode+2];
  double **charredmass=new double *[m_numnode+2];
  double **driedmass=new double *[m_numnode+2];
  double **moisture=new double *[m_numnode+2];
  double **S=new double *[m_numnode+2];
  double **oldS=new double *[m_numnode+2];

  double **AP=new double *[m_numnode+2];
  double **AW=new double *[m_numnode+2];
  double **AE=new double *[m_numnode+2];
  double **AN=new double *[m_numnode+2];
  double **AS=new double *[m_numnode+2];
  double **B=new double *[m_numnode+2];

  //Added 01Oct10 by asb*************************
  //double **water_heat_loss=new double *[m_numnode+2];
  // dwdt = rate of water loss
  double **dwdt_0=new double *[m_numnode+2];
  //holds the state of necrosis (0 to 1) for each node
  double **necrosis=new double *[m_numnode+2];
  
  //****************************************************

  int **dry=new int *[m_numnode+2];
  int **volatilizing=new int *[m_numnode+2];
  int **charred=new int *[m_numnode+2];

  // added 24 Sep2010 - asb ****************************
  double *cntrlvols=new double [m_numnode+2]; // -ec
  double dwdt=0.0; // water loss rate (kg/m3/s)
  double dpdt=0.0; // Rate of pyrolysis (kg/m3/s)
  double T_dummy=0; //temperature dummy for water loss function
  double t_sub; // time sub-interval for heat transfer loop - asb
  double nec_rate;// rate of tissue necrosis
//*********************************************************
  //Added 01Oct2010 - asb
  //double vapor_heat = 2461; // Heat of vaporiztion of water
  //double C_pyr = 1000; //Cp of pyrolisate [J/kg/K]
  double chardensfrac = 0.30; //Density fraction of char
  double total_water_loss = 0.0; //total mass of water lost (kg)
  double total_pyr_loss = 0.0; //total mass loss from pyrolysis
  double thermal_dummy;// holds minimum thermal diffusivity at surface. 
  double thermal_diff=1.0; // holds minimum thermal diffusivity at surface.
  int counter = 0;  //counter dummy
  double *pyr_heat_loss=new double [m_numtheta+1]; // heat loss by pyrolysis // -ec
  double *evap_heat_loss=new double [m_numtheta+1]; // heat loss by evaporation // -ec
  int *dry_node=new int [m_numtheta+1]; // current dry node for a given wedge   // -ec
  int *charred_node=new int [m_numtheta+1]; // current charred node for a given wedge //-ec
  double rho_dummy = 0; //density dummy
  double M_dummy = 0; // moisture dummy
  //double surf_vol = 0; //holds volume between surface nodes
   double Pi=M_PI; // defined in math.h
   double emissivity; // emissivity (and absorptivity) of stem surface at each wedge  // -ec
   //double **emissivity=new double *[m_numtheta+1];// emissivity (and absorptivity) of stem surface at each wedge
   int *necrotic_node=new int [m_numtheta+1];//hold most recent necrotic node for a wedge // -ec
   double *necrotic_depth=new double [m_numtheta+1]; //Current depth of necrosis for a wedge // -ec
   //double mean_nec_depth;
   double *char_depth=new double [m_numtheta+1];//current depth of charring for a wedge   // - ec
   double *int_heat_flux=new double [m_numtheta+1]; //time-integrated heat flux for a given wedge  // -ec
   double radinputval;
   double tempval;
//*****************************************************************
 
 
 int i;
  // Memory Allocation
  for(i=0;i<=(int)m_numnode+1;i++)
    {
      nodes[i]=new double [m_numtheta+1];
      temps[i]=new double [m_numtheta+1];
      Dx[i]=new double [m_numtheta+1];
      dx[i]=new double [m_numtheta+1];
      cntrlsrfcs[i]=new double [m_numtheta+1];
      rho[i]=new double [m_numtheta+1];
      kcs[i]=new double [m_numtheta+1];
      Cp[i]=new double [m_numtheta+1];
      k[i]=new double [m_numtheta+1];
      mass[i]=new double [m_numtheta+1];
      volatilemass[i]=new double [m_numtheta+1];
      hightemps[i]=new double [m_numtheta+1];
 
      oldV[i]=new double [m_numtheta+1];
      //	energyconservation[i]=new double [m_numtheta+1]; //commented for 2D
      charredmass[i]=new double [m_numtheta+1];
      driedmass[i]=new double [m_numtheta+1];
      moisture[i]=new double [m_numtheta+1];
      S[i]=new double [m_numtheta+1];
      oldS[i]=new double [m_numtheta+1];

      AP[i]=new double [m_numtheta+1];
      AW[i]=new double [m_numtheta+1];
      AE[i]=new double [m_numtheta+1];
      AN[i]=new double [m_numtheta+1];
      AS[i]=new double [m_numtheta+1];
      B[i]=new double [m_numtheta+1];

      dry[i]=new int [m_numtheta+1];
      charred[i]=new int [m_numtheta+1];
      volatilizing[i]=new int [m_numtheta+1];
      dwdt_0[i]=new double[m_numtheta+1];//added 01Oct10 by asb
     necrosis[i]=new double[m_numtheta+1];
    }

  printf("past memory allocation\n");

  int count;
  //int subcount=0;
  int incrad=0;
  int conifer=1;
  double CW=0;

  double radval=0;
  double netheat=0,
  //double Tflame=0,
     param1=.5,
    param2=.1667,
    param3=.066667,
    //lnk=0,
    //K=0,
    Ar=1.09*pow(10,24),
    E=166810;

  switch(m_speciesname)
    {
    case 0://Chestnut Oak
    if(rhobark<=0)  rhobark=742;//measured on site (average)
     if(rhowood<=0)  rhowood=573;//measured on site (average)
     expfactor=1;  // no use
      if(innerbark<=0) inoverout=2.89;
      param1=.99;
      param2=.5;
      param3=.2;
      m_flux=1000;   
      Ar=7.15*pow(10,34);
      E=235403;
      incrad=1; //no use
      conifer=0;//no use
      Tsat=372.23;//USFS/MeadWestvaco Vinton Furnace Experimental Forest (elevation 275 m)  //no use

      break;
    case 1://Douglas-fir
      if(rhobark<=0) rhobark=440;
      if(rhowood<=0) rhowood=520;//Simpson and TenWolde
      expfactor=1.75;
      if(innerbark<=0) inoverout=4;
      param1=.50;
      param2=.1667;
      param3=.0666667;
      m_flux=16000; //no use
      incrad=0;
      conifer=1;
      Tsat=369.77;//KMSO 3205 feet.
      Ar=1.44*pow(10,12); //no use
      E=95104; //no use
      break;
		
    case 2://Juvenile Red Maple
       if(rhobark<=0) rhobark=592;//measured on site (average)
        if(rhowood<=0)  rhowood=457;//measured on site (average)

      //The following line was necessary for the 2002 Ohio experiments because the only moisture measured
      //was whole bark or whole wood.  This converts the whole bark measurement into an inner bark value.
      //For input files which already provide inner bark moisture instead of whole bark moisture this line 
      //should be comented out.
      m_moisture=3*m_moisture/2.2;
      expfactor=1;
       if(innerbark<=0) inoverout=1.5;
      param1=1;
      param2=1;
      param3=.2;
      m_flux=16000;
      incrad=1;
      conifer=0;
      Tsat=372.23;
      Ar=1.09*pow(10,24);
      E=166810;
      break;
      
    case 3://Juvenile Chestnut Oak
     if(rhobark<=0)  rhobark=742;//measured on site (average)
     if(rhowood<=0) rhowood=573;//measured on site (average)
      expfactor=1;
      if(innerbark<=0) inoverout=1.25;
      param1=1;
      param2=1;
      param3=.2;
      m_flux=1000;
      Ar=7.15*pow(10,34);
      E=235403;
      incrad=1;
      conifer=0;
      Tsat=372.23;
      break;


    case 4://Ponderosa Pine
      if(rhobark<=0)    rhobark=260;//Dan 29 may 03
      if(rhowood<=0) rhowood=490;//Dan 29 may 03
      expfactor=1;
       if(innerbark<=0) inoverout=6;
      param1=.63;
      param2=.26;
      param3=.1852;
      m_flux=1000;
      incrad=0;
      conifer=1;
      Tsat=369.77;
      Ar=2.2*pow(10,4);
      E=46353;
      break;
      
    case 5://Red Maple
     if(rhobark<=0)  rhobark=592;//measured on site (averaged)
     if(rhowood<=0) rhowood=457;//measured on site (averaged)

      //The following line was necessary for the 2002 Ohio experiments because the only moisture measured
      //was whole bark or whole wood.  This converts the whole bark measurement into an inner bark value.
      //For input files which already provide inner bark moisture instead of whole bark moisture this line 
      //should be comented out.
      //m_moisture=3*m_moisture/2.2;
      expfactor=1;
       if(innerbark<=0) inoverout=1.5;
      param1=.99;
      param2=.5;
      param3=.2;
     //param3=1.0;
      m_flux=16000;
      incrad=1;
      conifer=0;
      Tsat=372.23;
      Ar=1.09*pow(10,24),
	E=166810;
      break;
      
     case 6://Hickory (Carya)
     if(rhobark<=0)  rhobark=(698+745)/2;//measured in lab
     if(rhowood<=0)  rhowood=705;//measured on site (average)
      expfactor=1;
       if(innerbark<=0)  inoverout=0.46;
      param1=1;
      param2=.83;
      param3=.26;
      m_flux=1000;   
      Ar=7.15*pow(10,34);
      E=235403;
      incrad=1;
      conifer=0;
      Tsat=373;
      break;
		
    default://Douglas-fir
     if(rhobark<=0)  rhobark=440;//440;
     if(rhowood<=0) rhowood=520;//Simpson and TenWolde
      expfactor=5;
      if(innerbark<=0) inoverout=4;
      param1=.50;
      param2=.1667;
      param3=.0666667;
      m_flux=16000;
      incrad=0;
      conifer=1;
      Tsat=369.77;
      Ar=1.44*pow(10,12);
      E=95104;
      break;
    }

/*
//Saturation (boiling) temp from original firestem code
  calculate Tsat based on elevation in feet.
  	pressure=6894.76*((6.591*pow(10,-9))*elevation*elevation+
  						(-5.2911*pow(10,-4))*elevation+
  							(1.4724*pow(10,1)));
  	Tsat=(-1.875*pow(10,-9))*pressure*pressure+
  			(6.593*pow(10,-4))*pressure+
  				(3.259*pow(10,2));
*/	
	
  //Temporary output files
  ofstream ooool("testing.txt");
  ofstream fp("stemtemp.xls");
  ofstream camb("cambium.out");
  ofstream surf("surface.out");

  ofstream debug("debug.txt");
  ofstream water("water.xls");
 // ofstream ftemp("ftemp.xls");
  ofstream tempmovie("T_Movie.xls");
  ofstream tcp_file("TCP.xls");
  ofstream nec_char("Necrosis_Char.xls");
  ofstream moisttrack("moist-track.xls");

  fp<<iterations<<"\t"<<seconds<<endl;
 
  //Set up node depths -- uses 'Dr' from constants at beginning of code.
  int thetacount;
  for(thetacount=1;thetacount<=m_numtheta;thetacount++){
    for(count=m_numnode;count>=1;count--){
      nodes[count][thetacount]=m_radius-(m_numnode-count)*Dr;
    }
  }
  //
  //control surfaces at each end and half way between the nodes - unit control surfaces
  //
  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    cntrlsrfcs[1][thetacount]=nodes[1][thetacount];
  }

  for(count=2;count<m_numnode+1;count++)
    {
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	cntrlsrfcs[count][thetacount]=(nodes[count-1][thetacount]+nodes[count][thetacount])/2;
	 }
    }

  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    cntrlsrfcs[m_numnode+1][thetacount]=nodes[m_numnode][thetacount];
  }

// Find node corresponding to thermocouple probe depth  - asb 03Oct10
 // int tcp=1;
    for(count=m_numnode;count>=1;count--)
      {
      	if(nodes[count][1]<=(m_radius-tcdepth)) 
	  {    	      tcloc = count;
			break;
	  }
       }
       
  int dis=1; //discontinuity flag
  int discontinuity; // the value of this node splits the wood and the bark

  for(count=m_numnode;count>=1;count--)
    {
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	//nodes[count][thetacount]=nodes[count-1][thetacount];
	if(nodes[count][thetacount]<m_radius-m_thickness && dis==1)
	  {
	    discontinuity = count;
	    dis=0;
	  }
      }
    }


  //initialize temperature array, set Dx and dx for each control volume
  for(count=1;count<m_numnode+1;count++)
    {
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {	
	temps[count][thetacount]=m_inittemp+273;
	B[count][thetacount]=m_inittemp+273;
	dry[count][thetacount]=0;
	charred[count][thetacount]=0;

	//Distance between 2 control volumes
	Dx[count][thetacount]=cntrlsrfcs[count+1][thetacount]-cntrlsrfcs[count][thetacount];

	//Distance between 2 nodes
	if(count<m_numnode)
	  dx[count][thetacount]=nodes[count+1][thetacount]-nodes[count][thetacount];

	// density of charred mass is approximated by multiplying 0.27 - mass = density*(unit)volume PI*R*R
	charredmass[count][thetacount]=.27*rhobark*Pi*(cntrlsrfcs[count+1][thetacount]*cntrlsrfcs[count+1][thetacount]-cntrlsrfcs[count][thetacount]*cntrlsrfcs[count][thetacount]);
	//tillman et al. say 73% dry mass (D-fir) is volatile on pp. 41, Table IV.
	driedmass[count][thetacount]=rhobark*Pi*(cntrlsrfcs[count+1][thetacount]*cntrlsrfcs[count+1][thetacount]-cntrlsrfcs[count][thetacount]*cntrlsrfcs[count][thetacount])*(1.0/m_numtheta);
	S[count][thetacount]=1;
	oldS[count][thetacount]=1;	
      }
    }

  //
  //set up initial densities, properties
  //

  int flag=0;
  int flag1=0;
  for(count=1;count<=m_numnode;count++)
    {

      for(thetacount=1;thetacount<=m_numtheta;thetacount++) 
	{	
	//Initialize necrosis and thermal params
        emissivity=0.85; // -ec
	necrotic_node[thetacount]=m_numnode+1; // -ec
	necrosis[count][thetacount]=1;   // -ec
	necrotic_depth[thetacount]=0.;  // -ec
	char_depth[thetacount]=0.;  // -ec

	//Initialize dry and charred node flags
	if(count==1)
	  {
	  dry_node[thetacount]=m_numnode +1;
	  charred_node[thetacount]=m_numnode+1;
	 
	  }
     

	// Thermophysical properties of the wood
	if(nodes[count][thetacount]<nodes[discontinuity][thetacount])
	  {
	    rho[count][thetacount]=rhowood*(((param1-param2)/nodes[discontinuity][thetacount]*nodes[count][thetacount]+param2)*m_moisture+1);
	    if(W_moisture>0){
	      rho[count][thetacount]=rhowood*(W_moisture+1);
	    }

	    
	  }
	

	// Thermophysical properties of the bark
	else
	  {
	    // inner bark (moisture profile is 100%)
	    // The variable 'inoverout' helps to identify the inner and outer bark
	    if(nodes[count][thetacount]<(m_thickness)/inoverout+nodes[discontinuity][thetacount])
	      {
		if(flag1==0)
		  {
		    cout<<"Internal bark :"<<count;
		    flag1=1;
		    cout<<endl;
		  }
		rho[count][thetacount]=rhobark*(1+m_moisture);
		 if(I_moisture>0){
	      rho[count][thetacount]=rhobark*(I_moisture+1);
		}
	      }

	    //outer bark (moisture profile is P3%)
	    else
	      {
		if(flag==0)
		  {
		    //cout<<"External bark :"<<count;
		    flag=1;
		    cout<<endl;
		  }
		rho[count][thetacount]=rhobark*(1+param3*m_moisture);
		if(O_moisture>0){
	      rho[count][thetacount]=rhobark*(O_moisture+1);
		}
	      }
	  }
      }
    }

  // Printing the Node, control surface, dx and DX values into debug.txt
    debug<<"Nodes list"<<endl;
    for(count=1;count<=m_numnode;count++){	
    debug<<count<<" ";
    }
    debug<<endl;
    
  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    for(count=1;count<=m_numnode;count++){	
      debug<<nodes[count][thetacount]<<" ";
    }
    debug<<endl;
  }
  debug<<endl;

  debug<<"cntrlsrfcs list"<<endl;
  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    for(count=1;count<=m_numnode+1;count++){	
      debug<<cntrlsrfcs[count][thetacount]<<" ";
    }
    debug<<endl;
  }
  debug<<endl;

  debug<<"dx list"<<endl;
  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    for(count=1;count<=m_numnode;count++){	
      debug<<dx[count][thetacount]<<" ";
    }
    debug<<endl;
  }
  debug<<endl;

  debug<<"Dx list"<<endl;
  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    for(count=1;count<=m_numnode;count++){	
      debug<<Dx[count][thetacount]<<" ";
    }
    debug<<endl;
  }
  debug<<endl;

  debug<<"temps list"<<endl;
  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
    for(count=1;count<=m_numnode;count++){	
      debug<<temps[count][thetacount]<<" ";
    }
    debug<<endl;
  }
  debug<<endl;

  // End of Debug output 

  //
  //all the initialization is done.
  //


  double dt=0;
  //double radinput=0;
  //double convinput=0;
  //double bet=0;
  //double maxtemp=0;
  double dtheta;
  double *totinput= new double [m_numtheta+1];
  double total_time=0;
  //int numiterations=0;
  //int greatbigcount=0; // used in necrosis routine ????  -asb
	
  //set up for fire loop
  dt=0.1; // initial value for time step -- this may adjust during simulation
  cout << "Time Step "<<dt<<endl;
  fp<<dt<<endl;	
  //int radio=0;
 
  // Set up the initial moisture according to the moisture profile mentioned in Section 3.2.1 of Josh's thesis
  // If Inner, Outer and Wood moistures are given in input file, they override the profile calculations below - asb
  for (count=1;count<m_numnode+1;count++)
  
    {
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) 
	{
       

		
	//outerbark
	//if(nodes[count][thetacount] > (nodes[m_numnode][thetacount]-nodes[discontinuity][thetacount])/inoverout + nodes[discontinuity][thetacount])
	 if(nodes[count][thetacount] >= ((nodes[m_numnode][thetacount]-nodes[discontinuity][thetacount])*(1-(1/(1+inoverout)))+nodes[discontinuity][thetacount]))
	   {
	    moisture[count][thetacount]=m_moisture*param3;
	    //Override the moisture calculation with the input value
	    if(O_moisture>0){
	      moisture[count][thetacount]=O_moisture;
	      
	    }
	  }
	//inner bark
	else
	  {
	    moisture[count][thetacount]=m_moisture; // m_moisture = I_moisture
	  }
	//wood
	if(count<=discontinuity)
	  {  
	    moisture[count][thetacount]=m_moisture*((param1-param2)/nodes[discontinuity][thetacount]*nodes[count][thetacount]+param2);
	        //Override the moisture calculation with the input value
	    if(W_moisture>0){
	      moisture[count][thetacount]=W_moisture;
		}
	  }
	}
    }
	

  // ****  Vector of volumes between control surfaces - asb 24 Sep 2010 ***********
  for(count=1;count<=m_numnode;count++){
    cntrlvols[count]=Pi*(cntrlsrfcs[count+1][1]*cntrlsrfcs[count+1][1]-cntrlsrfcs[count][1]*cntrlsrfcs[count][1])*(1.0/m_numtheta);
  }

  // Set up the mass of each of the nodes
  for(count=1;count<=m_numnode;count++)
    {
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	//wood
	if(nodes[count][thetacount]<nodes[discontinuity][thetacount])
	  {
	    //
	    //Gives declining linear moisture profile in wood (?)
	    mass[count][thetacount]=(((param1*m_moisture)-(param2*m_moisture))/nodes[discontinuity][thetacount]*nodes[count][thetacount]+(param2*m_moisture))*rhowood*(cntrlvols[count]); // -ec
	    volatilemass[count][thetacount]=.862*rhowood*(cntrlvols[count]); 
	    //the 86.2% comes from Tillman et al. pp 41, but it really doesn't matter because the wood almost never pyrolizes
	    //in the validation experiments it would not have come into play.  Any tree where it did would be clearly dead.
	  }

	//inner bark
	if(nodes[count][thetacount]>=nodes[discontinuity][thetacount])
	  {
	    mass[count][thetacount]=rhobark*m_moisture*(cntrlvols[count]);  
	    volatilemass[count][thetacount]=.73*rhobark*(cntrlvols[count]);
	  }

	//outer bark - this code overrides the previous if loop
	if(nodes[count][thetacount]>(m_thickness)/inoverout+nodes[discontinuity][thetacount])
	  {

	    mass[count][thetacount]=param3*rhobark*m_moisture*(cntrlvols[count]);
	    volatilemass[count][thetacount]=.73*rhobark*(cntrlvols[count]);
	  } 
		

	// Initial water loss rate - this is used as a baseline to check for water loss in the rate calculation below
	dwdt_0[count][thetacount]=W_mult*0.001*(k_water/sqrt(m_inittemp+273.15))*rho[count][thetacount]*moisture[count][thetacount]*exp(-E_wateroverR/(m_inittemp+273.15));
			
      }//theta count loop
    }//count loop



  //***********************************************************************************************************************************
  // ********************************************       MassTransfer Loop ************************************************************
  //***********************************************************************************************************************************
  
  // *************  numreads = number of times data are read from the input file *********************
  int numreads=0;
  int timestep_index = 0;
  int printflag=0;
  //int printflag2=0;
  t_sub=0.0;
  
  while(total_time <=seconds)  // Added to accomodate variable time step routine
    {	timestep_index = timestep_index + 1;

      
  
	 // Read data as each time step is reached (or passed). 
	 //Reset if all wedge data at a time step have been read.  - asb 02Oct10
	 if(t_sub >= t_step ||total_time<=0){
	    t_sub = t_sub-t_step;
	    numreads=numreads+1;
	    for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
		 bndycond>>totinput[thetacount];
		}
	  } // end of t_sub loop
	
      for(count=1;count<=m_numnode;count++)
	{
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	    //CW = heat capacity of water
	    //CW relation is from curve fitted to water Cp data from Incropera & DeWitt (2002) [kJ/kg*K] - asb
	   if(temps[count][thetacount]<630){
	      CW = 3.8 + 130/(645 - temps[count][thetacount]); 
	   }
	   else{
	     CW=15;
	   }
	   
	   // Dummy vars below just for convenience when used in calculations below
	     T_dummy=temps[count][thetacount];
	      M_dummy = moisture [count][thetacount];	
	   
	      if(dry[count][thetacount]==0){
	     
     	      //Assign densities
	      if(nodes[count][thetacount]<nodes[discontinuity][thetacount])
		{  rho_dummy = rhowood;
		}
	      else{
		rho_dummy = rhobark;
	      }
	
	//Change line below to 'if(count == dry_node[thetacount]-1)' to limit evap to first wet node.
	   if(count<=dry_node[thetacount]-1){
	     //dwdt = rate of water loss. Based on eq. 33 Morvan and Dupuy 2001 
	     //Combustion and Flame 127:1981-1994. Units are kg/m3/s	
	      dwdt=W_mult*0.001* (k_water/sqrt(T_dummy))*rho_dummy*M_dummy*exp(-E_wateroverR/T_dummy);
	      // dwdt=0; // uncomment to turn off water loss
		  if(dwdt>=dwdt_0[count][thetacount]){
		    //only rate above initial loss rate is used. This stops water loss when
		    //no stem heating is occuring (i.e., when stem is at To).
		      dwdt=dwdt - dwdt_0[count][thetacount];
	      	      mass[count][thetacount]=mass[count][thetacount]-(dwdt)*(cntrlvols[count])*dt;
		      //CW has units of kJ/kg*K for use in bark & wood Cp relations, so is converted to J/kg*K 
		      // here by mutliplying by 1000. ~asb
		      evap_heat_loss[thetacount]=evap_heat_loss[thetacount] + dwdt * (cntrlvols[count]) * dt * CW *1000 * T_dummy;
		      total_water_loss = total_water_loss + dwdt*(cntrlvols[count])*dt;
		      
		      //adjust wood moisture
		      if(nodes[count][thetacount]<nodes[discontinuity][thetacount])
			{ moisture[count][thetacount]=mass[count][thetacount]/(rhowood*(cntrlvols[count]));
		      }

		      //adjust bark moisture
		    if(nodes[count][thetacount]>=nodes[discontinuity][thetacount])
		      {moisture[count][thetacount]=mass[count][thetacount]/(rhobark*(cntrlvols[count]));
		    }
		      if(rho[count][thetacount]>=rho_dummy + (dwdt * dt)){
		          rho[count][thetacount]=rho[count][thetacount] - (dwdt*dt);
		      }  
		  		     
		} // end of If dwdt>0
	    }// end "first wet node" condition 

	    // To save time, the 2nd conditon keeps this loop from being executed more than once per node.
	   if(moisture[count][thetacount]<0.01&&count<dry_node[thetacount])   
	      {
	      moisture[count][thetacount]=0.0;
	      mass[count][thetacount]=0.0;
	      dry[count][thetacount]=1;
	      dry_node[thetacount]=count;
	      
	      	if(count<discontinuity)
		  rho[count][thetacount]=rhowood; 
		else
		  {
		    rho[count][thetacount]=rhobark;
		  }
	     } //end of low moisture limit
	     
	   }// end of  wet condition and dwdt loop
	   
	   //Pyrolysis
	    if(dry[count][thetacount]==1&&count==charred_node[thetacount]-1)  //charred[count][thetacount]==0)	
	      {CW=0;
	       	if(count<discontinuity)
		 rho_dummy=rhowood; 
		else
		  {
		    rho_dummy=rhobark;
		  }
		//dpdt has units of kg/m3/s	  
	      dpdt = P_mult * A_pyr*rho_dummy*(1- chardensfrac)*exp(-E_pyroverR/T_dummy);
	      //dpdt=0;
	      rho[count][thetacount] = rho[count][thetacount] - dpdt*dt;
	      total_pyr_loss = total_pyr_loss + dpdt * (cntrlvols[count]) * dt;
	      pyr_heat_loss[thetacount]=dpdt * (cntrlvols[count]) * dt * 1000 * T_dummy;// 1000 J/kg*K is approximate Cp of air or CO2 
	    if(rho[count][thetacount]<=(chardensfrac+.01)*rho_dummy){  // checks to see if charred mass is below limit
	      rho[count][thetacount]=chardensfrac*rho_dummy;
	      charred[count][thetacount]=1;
	      charred_node[thetacount]=count;   //-ec
	      char_depth[thetacount]=m_radius-nodes[count][thetacount];  
	      //if (count==m_numnode) *emissivity[thetacount] = 0.80;//emissivity from Ragland and Aerts 1991
	      if (count==m_numnode) emissivity = 0.80;//emissivity from Ragland and Aerts 1991
	    }
	    }// end of dry pyrolysis condition
	  else{
	    dpdt=0.;
	  }
	  
	  //*******************************NECROSIS*******************************************
	//if not at node just below necrotic node, don't execute this loop
	if (count==necrotic_node[thetacount]-1){
	      //nec_rate=exp(-E_necoverR*((1./T_dummy) - (1./T_crit)));
	      nec_rate=kB_over_h*T_dummy*exp((DeltaH*1000*(T_dummy/T_crit - 1.0)-b_comp*T_dummy)/(R_mol*T_dummy));
	      necrosis[count][thetacount]=necrosis[count][thetacount]*exp(-nec_rate*dt);
	      //necrosis[count][thetacount]=necrosis[count][thetacount]+nec_rate*dt;
	      if(necrosis[count][thetacount]<0.001){
		necrotic_node[thetacount]=count;
		necrotic_depth[thetacount]=m_radius - nodes[count][thetacount];
	      }
	      
	  } //end necrosis
	  //*********************************************************************************
	  
	    if(charred[count][thetacount]==0){
	    //
	    //Thermal properties of wood (if count<discontinuity) and then bark (otherwise) 
	    //
	    if(count<discontinuity)
	      {
		//from woods handbook ch 3.  M in their work is in percent, thus moisture=M/100 here.
		Cp[count][thetacount]=1000*( (.1031 + .003867*T_dummy+ moisture[count][thetacount]*CW)/(1+moisture[count][thetacount]) + 100*moisture[count][thetacount]*(-.06191+.000236*T_dummy-.000133*moisture[count][thetacount]*100) );
		k[count][thetacount]=.01864 + (.1941 + .004064*moisture[count][thetacount]*100)*(rho[count][thetacount]/rhowood);
				
	      }
	    else
	      {
		//from RE Martin Oct 1963
		//making k as const after initial round
		if(rho[count][thetacount]>=rhobark){
		k[count][thetacount]=.041868*(5.026*rho[count][thetacount]*.001+13.241*(rho[count][thetacount]-rhobark)*.001+.0078*(T_dummy-273.15)-.379);
		}
		//cout<<"count:"<<count<<"  thetacount:"<<thetacount<<endl;
		//cout<<"k[count][thetacount] :"<<k[count][thetacount]<<endl;
		if(isnan(k[count][thetacount])!=0){
		  //cout<<"Rho:"<<rho[count][thetacount]<<"  T:"<<T_dummy<<endl;
		  return 0;
		}
		if(moisture[count][thetacount]>=.27){
		  Cp[count][thetacount]=4186.8*(.264+.00116*(T_dummy-273.15)+(moisture[count][thetacount]*CW/4.19)+.0832);  
		}
		else{
		  Cp[count][thetacount]=4186.8*(.264+.00116*(T_dummy-273.15)+(moisture[count][thetacount]*CW/4.19)+(.3052*moisture[count][thetacount]));
		}
		//cout<<"moisture :"<<moisture[count][thetacount]<<endl;
               //cout<<"cp:"<<Cp[count][thetacount]<<endl;
	      }
	    } // if charred = 0
	    
	    //if charred
	    if(charred[count][thetacount]==1){
	    k[count][thetacount] = 0.052;
	    Cp[count][thetacount] = (1.39 + 0.00036 * T_dummy) * 1000; //Ragland & Aerts 1991
	    }
	    
	    	//*****Constant thermal property OVERRIDES -- uncomment for testing*******************
	/*	
		k[count][thetacount]=0.17;
		Cp[count][thetacount]=1269;
		rho[count][thetacount]=723;
	*/	
		
	    // Calculates minimum thermal diffusivity at surface nodes.  asb 01Oct10
	    if (count==m_numnode){ 
	      thermal_diff=10;
	      for(counter=1;counter<=m_numtheta;counter++){
		thermal_dummy =  k[count][counter] / (rho[count][counter] * Cp[count][counter]);
	        if(thermal_dummy<thermal_diff){
		thermal_diff = thermal_dummy;
	      }
	    }
	  }// end of min thermal diffusivity loop

	  }//end of theta count	
	}//end of count loop  Sets up thermal properties (k, Cp)  
	

	
	
     
    //***********************        Set new dt according to minimum thermal diffusivity  *************************
    // Assuming Fourier number <= 1/4
      dt =0.25 * pow(Dx[2][1],2) / thermal_diff;  
       dt=0.1; // Override automatic time step --Comment out if instability arises due to large time step
      	if(dt>t_step)  dt=t_step;
	if(dt<0.005) dt=0.005; //Arbitrary lower limit
       t_sub = t_sub + dt;
	total_time=total_time+dt;
      //     *************************************************************************************************

      //
      //set the conductivity at the control surfaces
      //

      for(count=2;count<m_numnode+1;count++)
	{
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	    kcs[count][thetacount]=(k[count-1][thetacount]+k[count][thetacount])/2;
	  }
	}

      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	kcs[1][thetacount]=k[1][thetacount];
	kcs[m_numnode+1][thetacount]=k[m_numnode][thetacount];
      }

      //
      //Generate all coefficients for the linear system
      //with m_numnode X m_numtheta unknowns and m_numnode X m_numtheta equations
      //	
		
      
      for(count=2;count<m_numnode;count++)
	{
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {

	    //dtheta=2*Pi*nodes[count][thetacount]/m_numtheta;	
	  dtheta=2*Pi / m_numtheta;

	    tempval=rho[count][thetacount]*Cp[count][thetacount]*(cntrlsrfcs[count+1][thetacount]*cntrlsrfcs[count+1][thetacount]-cntrlsrfcs[count][thetacount]*cntrlsrfcs[count][thetacount])*dtheta/(2);

	    //From Equation 10
	    AW[count][thetacount]=-kcs[count][thetacount]*(cntrlsrfcs[count][thetacount])*dt*dtheta/(2*dx[count-1][thetacount]);

	    //From Equation 9
	    AE[count][thetacount]=-kcs[count+1][thetacount]*(cntrlsrfcs[count+1][thetacount])*dt*dtheta/(2*dx[count][thetacount]);

	    //From Equation 12
	    if(thetacount==m_numtheta)
	      AS[count][thetacount]=-(kcs[count][1]/cntrlsrfcs[count][1])*dt*Dx[count][thetacount]/(2*dtheta);
	    else
	      AS[count][thetacount]=-(kcs[count][thetacount+1]/cntrlsrfcs[count][thetacount+1])*dt*Dx[count][thetacount]/(2*dtheta);

	    //From Equation 11
	    if(thetacount==1)
	      AN[count][thetacount]=-(kcs[count][m_numtheta]/cntrlsrfcs[count][m_numtheta])*dt*Dx[count][thetacount]/(2*dtheta);	
	    else
	      AN[count][thetacount]=-(kcs[count][thetacount-1]/cntrlsrfcs[count][thetacount-1])*dt*Dx[count][thetacount]/(2*dtheta);

	    //From Equation 8
	    AP[count][thetacount]=(-AW[count][thetacount]-AE[count][thetacount]-AN[count][thetacount]-AS[count][thetacount])+ tempval;

	    //From Equation 13
	    if(thetacount==m_numtheta)
	      B[count][thetacount]=(tempval+AW[count][thetacount]+AE[count][thetacount]+AN[count][thetacount]+AS[count][thetacount])*temps[count][thetacount]-(AW[count][thetacount]*temps[count-1][thetacount])-(AE[count][thetacount]*temps[count+1][thetacount])-(AN[count][thetacount]*temps[count][thetacount-1])-(AS[count][thetacount]*temps[count][1]);
	    else if (thetacount==1)
	      B[count][thetacount]=(tempval+AW[count][thetacount]+AE[count][thetacount]+AN[count][thetacount]+AS[count][thetacount])*temps[count][thetacount]-(AW[count][thetacount]*temps[count-1][thetacount])-(AE[count][thetacount]*temps[count+1][thetacount])-(AN[count][thetacount]*temps[count][m_numtheta])-(AS[count][thetacount]*temps[count][thetacount+1]);	
	    else
	      B[count][thetacount]=(tempval+AW[count][thetacount]+AE[count][thetacount]+AN[count][thetacount]+AS[count][thetacount])*temps[count][thetacount]-(AW[count][thetacount]*temps[count-1][thetacount])-(AE[count][thetacount]*temps[count+1][thetacount])-(AN[count][thetacount]*temps[count][thetacount-1])-(AS[count][thetacount]*temps[count][thetacount+1]);

	  }
	}

      //
      //axial boundary conditions at the center
      //

      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	//dtheta=2*Pi*nodes[1][thetacount]/m_numtheta;	
	dtheta=2*Pi/m_numtheta;
	tempval=rho[1][thetacount]*Cp[1][thetacount]*(cntrlsrfcs[1+1][thetacount]*cntrlsrfcs[1+1][thetacount]-cntrlsrfcs[1][thetacount]*cntrlsrfcs[1][thetacount])*dtheta/(2);

	//From Equation 17
	AW[1][thetacount]=0;

	//From Equation 16
	AE[1][thetacount]=0;

	//From Equation 19
	if(thetacount==m_numtheta)
	  AS[1][thetacount]=-(kcs[1][1]/cntrlsrfcs[1][1])*dt*Dx[1][thetacount]/(2*dtheta);
	else
	  AS[1][thetacount]=-(kcs[1][thetacount+1]/cntrlsrfcs[1][thetacount+1])*dt*Dx[1][thetacount]/(2*dtheta);

	//From Equation 18
	if(thetacount==1)
	  AN[1][thetacount]=-(kcs[1][m_numtheta]/cntrlsrfcs[1][m_numtheta])*dt*Dx[1][thetacount]/(2*dtheta);	
	else
	  AN[1][thetacount]=-(kcs[1][thetacount-1]/cntrlsrfcs[1][thetacount-1])*dt*Dx[1][thetacount]/(2*dtheta);

	//From Equation 15
	AP[1][thetacount]=(-AN[1][thetacount]-AS[1][thetacount])+ tempval;

	//From Equation 20
	if(thetacount==m_numtheta)
	  B[1][thetacount]=(tempval+AN[1][thetacount]+AS[1][thetacount])*temps[1][thetacount]-(AN[1][thetacount]*temps[1][thetacount-1])-(AS[1][thetacount]*temps[1][1]);
	else if (thetacount==1)
	  B[1][thetacount]=(tempval+AN[1][thetacount]+AS[1][thetacount])*temps[1][thetacount]-(AN[1][thetacount]*temps[1][m_numtheta])-(AS[1][thetacount]*temps[1][thetacount+1]);	
	else
	  B[1][thetacount]=(tempval+AN[1][thetacount]+AS[1][thetacount])*temps[1][thetacount]-(AN[1][thetacount]*temps[1][thetacount-1])-(AS[1][thetacount]*temps[1][thetacount+1]);


      }

      //
      //surface boundary conditions
      //
      if(m_Radio==1)
	{

	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {

	    //dtheta=2*Pi*nodes[m_numnode][thetacount]/m_numtheta;
	  	dtheta=2*Pi/m_numtheta;
	    tempval=rho[m_numnode][thetacount]*Cp[m_numnode][thetacount]*((cntrlsrfcs[m_numnode+1][thetacount]*cntrlsrfcs[m_numnode+1][thetacount])-(cntrlsrfcs[m_numnode][thetacount]*cntrlsrfcs[m_numnode][thetacount]))*dtheta/(2.);

	    //From Equation 10 
	    AW[m_numnode][thetacount]=-kcs[m_numnode][thetacount]*(cntrlsrfcs[m_numnode][thetacount])*dt*dtheta/(2*dx[m_numnode-1][thetacount]);
			
	    //From Equation 23
	    AE[m_numnode][thetacount]=0;
			
	    //From Equation 12
	    if(thetacount==m_numtheta)
	      AS[m_numnode][thetacount]=-(kcs[m_numnode][1]/cntrlsrfcs[m_numnode][1])*dt*Dx[m_numnode][thetacount]/(2*dtheta);
	    else
	      AS[m_numnode][thetacount]=-(kcs[m_numnode][thetacount+1]/cntrlsrfcs[m_numnode][thetacount+1])*dt*Dx[m_numnode][thetacount]/(2*dtheta);

	    //From Equation 11
	    if(thetacount==1)					
	      AN[m_numnode][thetacount]=-(kcs[m_numnode][m_numtheta]/cntrlsrfcs[m_numnode][m_numtheta])*dt*Dx[m_numnode][thetacount]/(2*dtheta);
	    else
	      AN[m_numnode][thetacount]=-(kcs[m_numnode][thetacount-1]/cntrlsrfcs[m_numnode][thetacount-1])*dt*Dx[m_numnode][thetacount]/(2*dtheta);

	    AP[m_numnode][thetacount]=-AW[m_numnode][thetacount]-AE[m_numnode][thetacount]-AN[m_numnode][thetacount]-AS[m_numnode][thetacount]+ tempval;

	    // Radiation Forcing applied to the surface node

	    radinputval = totinput[thetacount];

	    //From Equation 22.  1000 is a conversion factor from kW/m2 to W/m2
	    //netheat is net radiant heat flux. Note that emissivity = absorptivity
	    // netheat = *emissivity[thetacount] *( (radinputval)*1000-5.67E-8*(pow(temps[m_numnode][thetacount],4)-pow(m_inittemp+273.15,4))) ;
            netheat = emissivity *( (radinputval)*1000-5.67E-8*(pow(temps[m_numnode][thetacount],4)-pow(m_inittemp+273.15,4))) ;

	    //From Equation 21 - Note that the forcing, radval, is not a TEMPERATURE, it's the energy input over time dt.
    
	    radval = (2*Pi*nodes[m_numnode][thetacount]*(1.0/m_numtheta))*netheat*dt- pyr_heat_loss[thetacount]-evap_heat_loss[thetacount];
	    //Integrates heat flux for diagnostic purposes
	    if(radval>0)   int_heat_flux[thetacount] = int_heat_flux[thetacount] + radval/(2*Pi*nodes[m_numnode][thetacount]*(1.0/m_numtheta)); 
	

	    if(thetacount==m_numtheta)
	      B[m_numnode][thetacount]=(tempval+AW[m_numnode][thetacount]+AE[m_numnode][thetacount]+AN[m_numnode][thetacount]+AS[m_numnode][thetacount])*temps[m_numnode][thetacount]-(AW[m_numnode][thetacount]*temps[m_numnode-1][thetacount])-(AN[m_numnode][thetacount]*temps[m_numnode][thetacount-1])-(AS[m_numnode][thetacount]*temps[m_numnode][1])+radval;
	    else if (thetacount==1)
	      B[m_numnode][thetacount]=(tempval+AW[m_numnode][thetacount]+AE[m_numnode][thetacount]+AN[m_numnode][thetacount]+AS[m_numnode][thetacount])*temps[m_numnode][thetacount]-(AW[m_numnode][thetacount]*temps[m_numnode-1][thetacount])-(AN[m_numnode][thetacount]*temps[m_numnode][m_numtheta])-(AS[m_numnode][thetacount]*temps[m_numnode][thetacount+1])+radval;
	    else
	      B[m_numnode][thetacount]=(tempval+AW[m_numnode][thetacount]+AE[m_numnode][thetacount]+AN[m_numnode][thetacount]+AS[m_numnode][thetacount])*temps[m_numnode][thetacount]-(AW[m_numnode][thetacount]*temps[m_numnode-1][thetacount])-(AN[m_numnode][thetacount]*temps[m_numnode][thetacount-1])-(AS[m_numnode][thetacount]*temps[m_numnode][thetacount+1])+radval;
				
	    // Printing values for verification
	    //if((timestep_index%1==0||timestep_index%1000==2||timestep_index%1000==3||timestep_index%1000==4)&&printflag==0){
	    if(((timestep_index<1000&&timestep_index%10==0)||timestep_index%1000==0||total_time>=seconds&&(printflag==0)&& thetacount==1)){
	      printflag=1;

//            cout<<"Time: "<<total_time<<endl;	      
//            cout<<"dt :"<<dt<<endl;
// 	      cout<<"numreads :"<<numreads<<endl;
// 	      cout<<"TCP node :"<<tcloc<<endl;
// 	      cout<<"Inoverout: "<<inoverout<<endl;
// 	      cout<<"Thermal Diff :"<<thermal_diff<<endl;
// 	      //cout<<"discontinuity "<<discontinuity<<endl;
// 	      //cout<< "thetacount"<<thetacount<<endl;
// 	      cout<<"net heat :"<<netheat<<endl;
// 	      cout<<"Water heat loss: "<<evap_heat_loss[thetacount]<<endl;
// 	      cout<<"radval "<<radval<<endl;
// 	      cout<<"radinput :"<<radinputval<<endl;
// 	      cout<<"rho[m_numnode][1] "<<rho[m_numnode][thetacount]<<endl;
// 	      cout<<"Cp[m_numnode][1] "<<Cp[m_numnode][thetacount]<<endl;
// 	      //cout<<"tempval "<<tempval<<endl;
// 	      cout<<"AW[m_numnode][thetacount] "<<AW[m_numnode][thetacount]<<endl;
// 	      cout<<"AE[m_numnode][thetacount] "<<AE[m_numnode][thetacount]<<endl;
// 	      cout<<"AN[m_numnode][thetacount] "<<AN[m_numnode][thetacount]<<endl;
// 	      cout<<"AS[m_numnode][thetacount] "<<AS[m_numnode][thetacount]<<endl;
// 	      cout<<"AP[m_numnode][thetacount] "<<AP[m_numnode][thetacount]<<endl;
// 	      cout<<"temps[m_numnode-1][thetacount] "<<temps[m_numnode-1][thetacount]<<endl;
// 	      cout<<"temps[m_numnode+1][thetacount] "<<temps[m_numnode+1][thetacount]<<endl;
// 	      cout<<"temps[m_numnode][thetacount+1] "<<temps[m_numnode][thetacount+1]<<endl;
// 	      cout<<"temps[m_numnode][thetacount-1]"<<temps[m_numnode][m_numtheta]<<endl;
// 	      cout<<"temps[m_numnode][thetacount] "<<temps[m_numnode][thetacount]<<endl;
// 	      //cout<<"rho[m_numnode][thetacount] "<<rho[m_numnode][thetacount]<<endl;
// 	      cout<<"kcs[m_numnode-1][thetacount] "<<kcs[m_numnode-1][thetacount]<<endl;
// 	      cout<<"Cp[m_numnode][thetacount] "<<Cp[m_numnode][thetacount]<<endl;
// 	      cout<<"moisture[m_numnode][thetacount] "<<moisture[m_numnode][thetacount]<<endl;
	
// 	      cout<<"mass[m_numnode][thetacount] "<<mass[m_numnode][thetacount]<<endl;
// 	      cout<<"Tot water loss (kg) :"<<total_water_loss<<endl;
// 	      cout<<"k[discontinuity][thetacount] "<<k[discontinuity][thetacount]<<endl;
// 	      cout<<"k[m_numnode][thetacount] "<<k[m_numnode][thetacount]<<endl;
// 	      cout<<"B[m_numnode][1] calculation "<<B[m_numnode][1]<<endl;
// 	      cout<<"Dry Node :"<<dry_node[thetacount]<<endl;
// 	      cout<<"Charred depth :"<<char_depth[thetacount]<<endl;
// 	      cout<<"Necrotic depth :"<<necrotic_depth[thetacount]<<endl;
// 	      cout<<"Necrosis:"<<necrosis[m_numnode][thetacount]<<endl;
// 	      cout<<"Q :"<<int_heat_flux[thetacount]<<endl;
// 	      cout<<" "<<endl;
	    }
	 	     evap_heat_loss[thetacount]=0.0;
	 	    pyr_heat_loss[thetacount] = 0.0;
	  }// End of thetacount
   
	}

	
      // Solving the linear equation AX=B using UMFPACK
	
	
      int    n = m_numnode*m_numtheta;
      int    *Ap = new int[n+1];
      int    *Ai = new int[n*5-2*m_numtheta];
      double *Ax = new double[n*5-2*m_numtheta];
      double *b = new double[n];
      double *x = new double[n];

      // Fill B matrix - RHS	
	
      int b_index=0;
	
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	for(count=1;count<=m_numnode;count++) {
	  b[b_index++] = B[count][thetacount];
	}
      }
	
	
      // Fill Ap matrix - row starting element

      Ap[0]=0;
      Ap[1]=ROW_ELEMENTS-1;
      int ap_count=2;
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	if(thetacount==1)
	  count=3; // start filling from node 3 only for first wedge - node 1 and 2 already initialized
	else
	  count=1; // other wedges
	for(;count<=m_numnode;count++) {
	  if(count==1) {
	    Ap[ap_count] = Ap[ap_count-1]+ROW_ELEMENTS-1; // increments the value for the last row in previous wedge
	    ap_count++;
	  }
	  else if(count==2) {
	    Ap[ap_count] = Ap[ap_count-1]+ROW_ELEMENTS-1; // increments the value for the first row of this wedge
	    ap_count++;
	  }
	  else {
	    Ap[ap_count] = Ap[ap_count-1]+ROW_ELEMENTS;
	    ap_count++;
	  }
	}
      }
      Ap[n]=Ap[n-1]+ROW_ELEMENTS-1; // very last row in the matrix

      // Printing B and Ap matrices into debug.txt

      int j;
      if(timestep_index==1) { // change the timestep_index to print the values into debug.txt
	debug << "B_index val "<<b_index<<endl;

	debug<<"Printing B"<<endl;
	for(j=0;j<n;j++) {
	  debug<<b[j]<<" ";
	  if(j%m_numnode==0)
	    debug<<endl;
	}
	debug<<endl;
	debug << "ap_index val "<<ap_count<<endl;		
	debug<<"Printing Ap"<<endl;
	for(j=0;j<n+1;j++) {
	  debug<<Ap[j]<<" ";
	  if(j%m_numnode==0)
	    debug<<endl;
	}
	debug<<endl;
      }
	
      // END - Printing B and Ap matrices
	
      // Fill Ax matrix - non-zero elements(data) and Ai matrix - column index

      int ax_count=0;
      int ai_count=0;
      int ap_index=0,as_index=m_numnode,an_index=m_numnode*(m_numtheta-1);
      //ae_index=1
      //aw_index=-1
      int an_indexflag=0,as_indexflag=0;
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	for(count=1;count<=m_numnode;count++) {
	  if(count==1 && thetacount==1){ // First row in First block - a block comprises the radial nodes in a wedge
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AE[count][thetacount];
	    Ax[ax_count++] = AS[count][thetacount];
	    Ax[ax_count++] = AN[count][thetacount];	

	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+1;
	    Ai[ai_count++] = as_index;
	    Ai[ai_count++] = an_index;
	  }
	  else if(thetacount==1 && count==m_numnode){ // Last row in First block
	    Ax[ax_count++] = AW[count][thetacount];	
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AS[count][thetacount];
	    Ax[ax_count++] = AN[count][thetacount];	
	
	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = as_index+=1;
	    Ai[ai_count++] = an_index+=1;
	  }
	  else if(thetacount==1){ // Intermediate rows of First block
	    Ax[ax_count++] = AW[count][thetacount];	
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AE[count][thetacount];
	    Ax[ax_count++] = AS[count][thetacount];
	    Ax[ax_count++] = AN[count][thetacount];	
	
	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = ap_index+1;
	    Ai[ai_count++] = as_index+=1;
	    Ai[ai_count++] = an_index+=1;
	  }
	  else if(thetacount>1 && thetacount<m_numtheta && count==1) { // First row in Intermediate Blocks
	    Ax[ax_count++] = AN[count][thetacount];
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AE[count][thetacount];	
	    Ax[ax_count++] = AS[count][thetacount];	
			
	    if(an_indexflag==0){
	      an_index=-1;
	      an_indexflag=1;
	    }

	    Ai[ai_count++] = an_index+=1;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = ap_index+1;
	    Ai[ai_count++] = as_index+=1;
	  }
	  else if(thetacount>1 && thetacount<m_numtheta && count==m_numnode) { // Last row in Intermediate Blocks
	    Ax[ax_count++] = AN[count][thetacount];
	    Ax[ax_count++] = AW[count][thetacount];
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AS[count][thetacount];	

	    Ai[ai_count++] = an_index+=1;
	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = as_index+=1;
	  }
	  else if(thetacount>1 && thetacount<m_numtheta) { // Intermediate rows of Intermediate Blocks
	    Ax[ax_count++] = AN[count][thetacount];
	    Ax[ax_count++] = AW[count][thetacount];
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AE[count][thetacount];	
	    Ax[ax_count++] = AS[count][thetacount];	

	    Ai[ai_count++] = an_index+=1;
	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = ap_index+1;
	    Ai[ai_count++] = as_index+=1;
	  }
	  else if(thetacount==m_numtheta && count==1){ // First row in Last Block
	    Ax[ax_count++] = AS[count][thetacount];
	    Ax[ax_count++] = AN[count][thetacount];
	    Ax[ax_count++] = AP[count][thetacount];	
	    Ax[ax_count++] = AE[count][thetacount];
		
	    if(as_indexflag==0){
	      as_index=-1;
	      as_indexflag=1;
	    }
			
	    Ai[ai_count++] = as_index+=1;
	    Ai[ai_count++] = an_index+=1;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = ap_index+1;
	  }
	  else if(thetacount==m_numtheta && count==m_numnode){ // Last row in Last Block
	    Ax[ax_count++] = AS[count][thetacount];
	    Ax[ax_count++] = AN[count][thetacount];
	    Ax[ax_count++] = AW[count][thetacount];
	    Ax[ax_count++] = AP[count][thetacount];	
			
	    Ai[ai_count++] = as_index+=1;
	    Ai[ai_count++] = an_index+=1;
	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+=1;
	  }
	  else if(thetacount==m_numtheta){ // Intermediate rows of Last Block
	    Ax[ax_count++] = AS[count][thetacount];	
	    Ax[ax_count++] = AN[count][thetacount];
	    Ax[ax_count++] = AW[count][thetacount];
	    Ax[ax_count++] = AP[count][thetacount];
	    Ax[ax_count++] = AE[count][thetacount];	

	    Ai[ai_count++] = as_index+=1;
	    Ai[ai_count++] = an_index+=1;
	    Ai[ai_count++] = ap_index;
	    Ai[ai_count++] = ap_index+=1;
	    Ai[ai_count++] = ap_index+1;
			
	  }
			
	}
      }
	
      // Printing Ai and Ax matrices into debug.txt
	
      int k,m;
      if(timestep_index%20==0||total_time>=seconds||timestep_index==1) {
	debug << "ax_count val "<<ai_count<<endl;	
	debug << "ai_count val "<<ax_count<<endl;	
	debug<<"Printing Ax"<<endl;
	j=0;
	for(k=0;k<m_numtheta;k++) {
	  debug<<Ax[j]<<" "<<Ax[j+1]<<" "<<Ax[j+2]<<" "<<Ax[j+3]<<" ";
	  debug<<endl;
	  j+=4;
	  for(m=0;m<m_numnode-2;m++) {
	    debug<<Ax[j]<<" "<<Ax[j+1]<<" "<<Ax[j+2]<<" "<<Ax[j+3]<<" "<<Ax[j+4]<<" ";
	    debug<<endl;
	    j+=5;
	  }
	  debug<<Ax[j]<<" "<<Ax[j+1]<<" "<<Ax[j+2]<<" "<<Ax[j+3]<<" ";
	  debug<<endl;
	  j+=4;
		
	}
		
      }

      if(timestep_index%20==0||total_time>=seconds||timestep_index==1) {
	debug<<"Printing Ai"<<endl;
	j=0;
	for(k=0;k<m_numtheta;k++) {
	  debug<<Ai[j]<<" "<<Ai[j+1]<<" "<<Ai[j+2]<<" "<<Ai[j+3]<<" ";
	  debug<<endl;
	  j+=4;
	  for(m=0;m<m_numnode-2;m++) {
	    debug<<Ai[j]<<" "<<Ai[j+1]<<" "<<Ai[j+2]<<" "<<Ai[j+3]<<" "<<Ai[j+4]<<" ";
	    debug<<endl;
	    j+=5;
	  }
	  debug<<Ai[j]<<" "<<Ai[j+1]<<" "<<Ai[j+2]<<" "<<Ai[j+3]<<" ";
	  debug<<endl;
	  j+=4;
		
	}
      }

      // END - Printing Ai and Ax matrices

      // Sparse matrix solver - UMFPACK - http://www.cise.ufl.edu/research/sparse/umfpack/

      double *null = (double *) NULL ;
      void *Symbolic, *Numeric ;
      (void) umfpack_di_symbolic (n, n, Ap, Ai, Ax, &Symbolic, null, null) ;
      (void) umfpack_di_numeric (Ap, Ai, Ax, Symbolic, &Numeric, null, null) ;
      umfpack_di_free_symbolic (&Symbolic) ;
      (void) umfpack_di_solve (UMFPACK_At, Ap, Ai, Ax, x, b, Numeric, null, null) ; //UMFPACK_At is used for row-major order of matrix A
      umfpack_di_free_numeric (&Numeric) ;

      ///////////////////////////

      //energyconservation[1]=-kcs[m_numnode]*(temps[m_numnode]-temps[m_numnode-1])/(dx[m_numnode-1]); // commented for 2D
      
      
      //***********************************************************************************
      //**************************** UPDATE TEMPERATURES ************************
      //**********************************************************************************
      int x_index=0;
      for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	for(count=1;count<=m_numnode;count++){
	  temps[count][thetacount]=x[x_index++];
	  	  //Limit wet nodes to Tsat - comment out if desired
		/*  if(moisture[count][thetacount]>0.29 && temps[count][thetacount]>Tsat){      //&&count!=dry_node[thetacount]-1){
		temps[count][thetacount]=Tsat;
		} */
	      if(temps[count][thetacount]<(m_inittemp+273.15)){
		temps[count][thetacount]=m_inittemp + 273.15;
	      }
	}
      }
  //***********************************************************************************
  
      // changed ftemp to every 1000 iterations 24 sep 2010 asb
     // if(timestep_index%1000==0||total_time>=seconds||timestep_index==1)
	//{
	  //for(thetacount=1;thetacount<=1/*m_numtheta*/;thetacount++) { // Printing just the temperature values of the 0th wedge
	    //ftemp<<(dt*timestep_index)<<" ";
	    //for(count=1;count<=m_numnode;count++){
	     // ftemp<<temps[count][thetacount]<<" ";
	    //}
	    //ftemp<<endl;
	 // }
	//}

//*************************Output files below this point *******************************
 
    if(timestep_index%20==0||total_time>=seconds||timestep_index==1) // Printing temps in all wedges for movie file
	{
	  tempmovie<<endl;

	if(timestep_index==1){
	  tempmovie<<" "; 
	    { thetacount=1;
	      for(count=1;count<=m_numnode;count++){
		tempmovie<<nodes[count][thetacount]<<" ";
	      }
	      tempmovie<<endl;
	    }
	}
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	    for(count=1;count<=m_numnode;count++){
	      if(count==1){
		tempmovie<<total_time<<" ";
	      }
	      tempmovie<<temps[count][thetacount]<<" ";
	    }
	    tempmovie<<endl;

	  }
	}
	
	    if(timestep_index%50==0||total_time>=seconds||timestep_index==1){ // Printing  water file
	if (timestep_index==1) water<<"Time_(s) Tot_Water_Loss_(kg)_ Tot_Pyrolized_Mass_(kg)"<<endl; 
	 water<<total_time<<" "<<total_water_loss<<" "<<total_pyr_loss<<endl; 
	 }
//}
	

// ***********   Output temperatures at TCP depth  - asb 03Oct10	***********************
	 if(timestep_index%50==0||total_time>=seconds||timestep_index==1) // Printing all wedges for movie file
	{
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	    if(thetacount==1){
	      tcp_file<<total_time<<" ";
	    }
	    tcp_file<<temps[tcloc][thetacount]<<" ";
	      }
	    tcp_file<<endl;

	  }
	
	

	 if(timestep_index%50==0||total_time>=seconds||timestep_index==1) // Printing all wedges for movie file
	 {
	   // if (timestep_index==1) nec_char<<"Time_(s) Necrotic_Depth_(m) Char_Depth_(m)"<<endl;
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) {
	    	nec_char<<total_time<<","<<necrotic_depth[thetacount]<<","<<char_depth[thetacount]<<endl;
	      }
	  // nec_char<<endl;
	  }


      if(timestep_index%1000==0||total_time>=seconds||timestep_index==1)
	{
	  for(thetacount=1;thetacount<=m_numtheta;thetacount++) { // Printing the moisture values(in %) for every 1000th iteration
	    for(count=1;count<=m_numnode;count++){	
	      moisttrack<<moisture[count][thetacount]<<" ";
	    }
	    moisttrack<<endl;
	  }
	  moisttrack<<endl;
	}

      // Free the memory associated with the sparse matrix - otherwise memory leak
      delete Ap;
      delete Ax;
      delete Ai;
      delete x;
      delete b;


      //********************************************************************************
    } //***************end of timestep_index (heat transfer ) loop *******************
     //*********************************************************************************

  	
  // Finalize everything to shut down the dialog box w/o any loose ends.
  //	

  for(i=0;i<=(int)m_numnode+1;i++)
    {
      delete [] nodes[i];
      nodes[i]=NULL;	
      delete [] temps[i];
      temps[i]=NULL;	
      delete [] Dx[i];
      Dx[i]=NULL;	
      delete [] dx[i];
      dx[i]=NULL;	
      delete [] cntrlsrfcs[i];
      cntrlsrfcs[i]=NULL;	
      delete [] kcs[i];
      kcs[i]=NULL;
      delete [] rho[i];
      rho[i]=NULL;
      delete [] Cp[i];
      Cp[i]=NULL;
      delete [] k[i];
      k[i]=NULL;
      delete [] mass[i];
      mass[i]=NULL;
      delete [] volatilemass[i];
      volatilemass[i]=NULL;
      delete [] hightemps[i];
      hightemps[i]=NULL;
      delete [] oldV[i];
      oldV[i]=NULL;
      delete [] dry[i];
      dry[i]=NULL;
      delete [] dwdt_0[i];
      dwdt_0[i]=NULL;	
      delete [] necrosis[i];
      necrosis[i]=NULL;


      delete [] volatilizing[i];
      volatilizing[i]=NULL;
      delete [] charred[i];
      charred[i]=NULL;
      delete [] charredmass[i];
      charredmass[i]=NULL;
      delete [] driedmass[i];
      driedmass[i]=NULL;
      delete [] moisture[i];
      moisture[i]=NULL;
      delete [] S[i];
      S[i]=NULL;
      delete [] oldS[i];
      oldS[i]=NULL;
      delete [] AP[i];
      AP[i]=NULL;
      delete [] AW[i];
      AW[i]=NULL;
      delete [] AE[i];
      AE[i]=NULL;
      delete [] AN[i];
      AN[i]=NULL;
      delete [] AS[i];
      AS[i]=NULL;
      delete [] B[i];
      B[i]=NULL;

    }


  delete [] nodes;
  delete [] temps;
  delete [] Dx;
  delete [] dx;
  delete [] cntrlsrfcs;
  delete [] kcs;
  delete [] rho;
  delete [] Cp;
  delete [] k;
  delete [] AP;
  delete [] AW;
  delete [] AE;
  delete [] AN;
  delete [] AS;
  delete [] B;

  delete [] mass;
  delete [] volatilemass;
  delete [] hightemps;
  delete [] oldV;
  delete [] dry;

  delete [] volatilizing;
  delete [] charred;
  //	delete [] energyconservation;
  delete [] charredmass;
  delete [] driedmass;
  delete [] moisture;
  delete [] S;
  delete [] oldS;
  delete [] dwdt_0;
   delete [] necrosis;

    
   
   
  fp.close();
  ooool.close();
  surf.close();
  camb.close();
  bndycond.close();
  moisttrack.close();
 // ftemp.close();
 nec_char.close();
  water.close();
  tempmovie.close();
  //	moist.close();
  debug.close();
  return 0;


    }

